@@ -45,6 +45,7 @@ struct MathButtonsPrivate
     GtkWidget *shift_left_menu, *shift_right_menu;
 
     GtkWidget *function_menu;
+    GtkWidget *const_menu;
 
     GList *superscript_toggles;
     GList *subscript_toggles;
@@ -60,10 +61,10 @@ struct MathButtonsPrivate
 
 G_DEFINE_TYPE_WITH_PRIVATE (MathButtons, math_buttons, GTK_TYPE_BOX);
 
-#define UI_BASIC_FILE       UI_DIR "/buttons-basic.ui"
-#define UI_ADVANCED_FILE    UI_DIR "/buttons-advanced.ui"
-#define UI_FINANCIAL_FILE   UI_DIR "/buttons-financial.ui"
-#define UI_PROGRAMMING_FILE UI_DIR "/buttons-programming.ui"
+#define UI_BASIC_RESOURCE_PATH       "/org/mate/calculator/ui/buttons-basic.ui"
+#define UI_ADVANCED_RESOURCE_PATH    "/org/mate/calculator/ui/buttons-advanced.ui"
+#define UI_FINANCIAL_RESOURCE_PATH   "/org/mate/calculator/ui/buttons-financial.ui"
+#define UI_PROGRAMMING_RESOURCE_PATH "/org/mate/calculator/ui/buttons-programming.ui"
 
 #define GET_WIDGET(ui, name) \
           GTK_WIDGET(gtk_builder_get_object((ui), (name)))
@@ -125,6 +126,9 @@ static ButtonData button_data[] = {
     {"function",           NULL, FUNCTION,
       /* Tooltip for the additional functions button */
       N_("Additional Functions")},
+    {"const",           NULL, FUNCTION,
+      /* Tooltip for the additional constant button */
+      N_("Additional constants")},
     {"x_pow_y",            "^", FUNCTION,
       /* Tooltip for the exponent button */
       N_("Exponent [^ or **]")},
@@ -319,7 +323,7 @@ set_data(GtkBuilder *ui, const gchar *object_name, const gchar *name, const char
 static void
 set_int_data(GtkBuilder *ui, const gchar *object_name, const gchar *name, gint value)
 {
-    GObject *object;  
+    GObject *object;
     object = gtk_builder_get_object(ui, object_name);
     if (object)
         g_object_set_data(object, name, GINT_TO_POINTER(value));
@@ -365,7 +369,7 @@ update_bit_panel(MathButtons *buttons)
 
     if (!buttons->priv->bit_panel)
         return;
-  
+
     enabled = math_equation_get_number(buttons->priv->equation, &x);
 
     if (enabled) {
@@ -381,21 +385,21 @@ update_bit_panel(MathButtons *buttons)
 
     gtk_widget_set_sensitive(buttons->priv->bit_panel, enabled);
     gtk_widget_set_sensitive(buttons->priv->base_label, enabled);
-      
+
     if (!enabled)
         return;
 
     for (i = 0; i < MAXBITS; i++) {
         const gchar *label;
 
-        if (bits & (1LL << (MAXBITS-i-1)))
+        if (bits & (1LLU << (MAXBITS-i-1)))
             label = " 1";
         else
             label = " 0";
         gtk_label_set_text(GTK_LABEL(buttons->priv->bit_labels[i]), label);
     }
 
-    base = math_equation_get_base(buttons->priv->equation);      
+    base = math_equation_get_base(buttons->priv->equation);
     label = g_string_new("");
     if (base != 8) {
         if (label->len != 0)
@@ -449,7 +453,7 @@ base_changed_cb(MathEquation *equation, GParamSpec *spec, MathButtons *buttons)
     GtkTreeModel *model;
     GtkTreeIter iter;
     gboolean valid;
-  
+
     if (buttons->priv->mode != PROGRAMMING)
         return;
 
@@ -478,7 +482,7 @@ load_mode(MathButtons *buttons, ButtonMode mode)
     GtkBuilder *builder, **builder_ptr;
     gint i;
     gchar *name;
-    const gchar *builder_file;
+    const gchar *path;
     static gchar *objects[] = { "button_panel", "character_code_dialog", "currency_dialog",
                                 "ctrm_dialog", "ddb_dialog", "fv_dialog", "gpm_dialog",
                                 "pmt_dialog", "pv_dialog", "rate_dialog", "sln_dialog",
@@ -490,32 +494,32 @@ load_mode(MathButtons *buttons, ButtonMode mode)
     default:
     case BASIC:
         builder_ptr = &buttons->priv->basic_ui;
-        builder_file = UI_BASIC_FILE;
+        path = UI_BASIC_RESOURCE_PATH;
         panel = &buttons->priv->bas_panel;
         break;
     case ADVANCED:
         builder_ptr = &buttons->priv->advanced_ui;
-        builder_file = UI_ADVANCED_FILE;
+        path = UI_ADVANCED_RESOURCE_PATH;
         panel = &buttons->priv->adv_panel;
         break;
     case FINANCIAL:
         builder_ptr = &buttons->priv->financial_ui;
-        builder_file = UI_FINANCIAL_FILE;
+        path = UI_FINANCIAL_RESOURCE_PATH;
         panel = &buttons->priv->fin_panel;
         break;
     case PROGRAMMING:
         builder_ptr = &buttons->priv->programming_ui;
-        builder_file = UI_PROGRAMMING_FILE;
+        path = UI_PROGRAMMING_RESOURCE_PATH;
         panel = &buttons->priv->prog_panel;
         break;
     }
-  
+
     if (*panel)
-        return *panel;
+        goto out;
 
     builder = *builder_ptr = gtk_builder_new();
     // FIXME: Show dialog if failed to load
-    gtk_builder_add_objects_from_file(builder, builder_file, objects, &error);
+    gtk_builder_add_objects_from_resource(builder, path, objects, &error);
     if (error) {
         g_warning("Error loading button UI: %s", error->message);
         g_clear_error(&error);
@@ -540,7 +544,7 @@ load_mode(MathButtons *buttons, ButtonMode mode)
 
         if (button_data[i].tooltip)
             gtk_widget_set_tooltip_text(button, _(button_data[i].tooltip));
-      
+
         atk_object_set_name(gtk_widget_get_accessible(button), button_data[i].widget_name);
     }
 
@@ -570,7 +574,7 @@ load_mode(MathButtons *buttons, ButtonMode mode)
         buffer[len] = '\0';
         gtk_button_set_label(GTK_BUTTON(widget), buffer);
     }
-  
+
     widget = GET_WIDGET(builder, "calc_superscript_button");
     if (widget) {
         buttons->priv->superscript_toggles = g_list_append(buttons->priv->superscript_toggles, widget);
@@ -662,7 +666,8 @@ load_mode(MathButtons *buttons, ButtonMode mode)
     gtk_builder_connect_signals(builder, buttons);
 
     display_changed_cb(buttons->priv->equation, NULL, buttons);
-  
+
+out:
     return *panel;
 }
 
@@ -679,7 +684,7 @@ converter_changed_cb(MathConverter *converter, MathButtons *buttons)
     }
     else {
         math_equation_set_source_units(buttons->priv->equation, unit_get_name(from_unit));
-        math_equation_set_target_units(buttons->priv->equation, unit_get_name(to_unit));      
+        math_equation_set_target_units(buttons->priv->equation, unit_get_name(to_unit));
     }
 
     g_object_unref(from_unit);
@@ -698,7 +703,7 @@ load_buttons(MathButtons *buttons)
     if (!buttons->priv->converter) {
         buttons->priv->converter = math_converter_new(buttons->priv->equation);
         g_signal_connect(buttons->priv->converter, "changed", G_CALLBACK(converter_changed_cb), buttons);
-        gtk_box_pack_start(GTK_BOX(buttons), GTK_WIDGET(buttons->priv->converter), FALSE, TRUE, 0);      
+        gtk_box_pack_start(GTK_BOX(buttons), GTK_WIDGET(buttons->priv->converter), FALSE, TRUE, 0);
     }
 
     panel = load_mode(buttons, buttons->priv->mode);
@@ -720,12 +725,12 @@ void
 math_buttons_set_mode(MathButtons *buttons, ButtonMode mode)
 {
     g_return_if_fail(buttons != NULL);
- 
+
     if (buttons->priv->mode == mode)
         return;
 
     buttons->priv->mode = mode;
-  
+
     if (mode == PROGRAMMING)
         math_equation_set_base(buttons->priv->equation, buttons->priv->programming_base);
     else
@@ -797,7 +802,7 @@ G_MODULE_EXPORT
 void
 subtract_cb(GtkWidget *widget, MathButtons *buttons)
 {
-    math_equation_insert_subtract(buttons->priv->equation);  
+    math_equation_insert_subtract(buttons->priv->equation);
 }
 
 
@@ -875,7 +880,7 @@ memory_cb(GtkWidget *widget, MathButtons *buttons)
     popup = math_variable_popup_new(buttons->priv->equation);
     gtk_window_set_transient_for(GTK_WINDOW(popup), GTK_WINDOW(gtk_widget_get_toplevel(widget)));
 
-    gtk_widget_get_allocation(widget, &allocation); 
+    gtk_widget_get_allocation(widget, &allocation);
     gdk_window_get_root_coords(gtk_widget_get_window(widget), allocation.x, allocation.y, &x, &y);
     gtk_window_move(GTK_WINDOW(popup), x, y);
     gtk_widget_show(GTK_WIDGET(popup));
@@ -964,7 +969,7 @@ shift_right_cb(GtkWidget *widget, MathButtons *buttons)
         }
     }
 
-    popup_button_menu(widget, GTK_MENU(buttons->priv->shift_right_menu));  
+    popup_button_menu(widget, GTK_MENU(buttons->priv->shift_right_menu));
 }
 
 
@@ -983,10 +988,10 @@ function_cb(GtkWidget *widget, MathButtons *buttons)
     if (!buttons->priv->function_menu) {
         gint i;
         GtkWidget *menu;
-        struct 
+        struct
         {
             gchar *name, *function;
-        } functions[] = 
+        } functions[] =
         {
             { /* Tooltip for the integer component button */
               N_("Integer Component"), "int " },
@@ -1008,7 +1013,7 @@ function_cb(GtkWidget *widget, MathButtons *buttons)
 
         for (i = 0; functions[i].name != NULL; i++) {
             GtkWidget *item;
-          
+
             item = gtk_menu_item_new_with_label(_(functions[i].name));
             g_object_set_data(G_OBJECT(item), "function", g_strdup(functions[i].function));
             gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
@@ -1017,9 +1022,67 @@ function_cb(GtkWidget *widget, MathButtons *buttons)
         }
     }
 
-    popup_button_menu(widget, GTK_MENU(buttons->priv->function_menu));  
+    popup_button_menu(widget, GTK_MENU(buttons->priv->function_menu));
 }
 
+static void
+insert_const_cb(GtkWidget *widget, MathButtons *buttons)
+{
+    math_equation_insert(buttons->priv->equation, g_object_get_data(G_OBJECT(widget), "const"));
+}
+
+
+void const_cb(GtkWidget *widget, MathButtons *buttons);
+G_MODULE_EXPORT
+void
+const_cb(GtkWidget *widget, MathButtons *buttons)
+{
+    if (!buttons->priv->const_menu) {
+        gint i;
+        GtkWidget *menu;
+        struct
+        {
+            gchar *name, *constant, *tooltip;
+        } constants[] =
+        {
+            { /* Tooltip for the c₀ component button */
+              N_("Velocity of Light"), "c₀", N_("299,792,458 m/s") },
+            { /* Tooltip for the μ₀ component button */
+              N_("Magnetic constant"), "μ₀", N_("1.2566370614×10⁻⁶ N/A²") },
+            { /* Tooltip for the ε₀ button */
+              N_("Electric constant"), "ε₀", N_("8.85418782×10⁻¹² s⁴A²/m³kg") },
+            { /* Tooltip for the G button */
+              N_("Newtonian constant of gravitation"), "G",  N_("6.67408×10⁻¹¹ m³/(s²kg)") },
+            { /* Tooltip for the h button */
+              N_("Planck constant"), "h", N_("6.62607004×10⁻³⁴ m²kg/s") },
+            { /* Tooltip for the ｅ button */
+              N_("Elementary charge"), "ｅ", N_("1.6021766208(98)×10⁻¹⁹ C") },
+            { /* Tooltip for the mₑ button */
+              N_("Electron mass"), "mₑ", N_("9.10938356×10⁻³¹ kg") },
+            { /* Tooltip for the mₚ button */
+              N_("Proton mass"), "mₚ", N_("1.672621898(21)×10⁻²⁷ kg") },
+            { /* Tooltip for the Nₐ button */
+              N_("Avogadro constant"), "Nₐ", N_("6.02214086×10²³ mol⁻¹") },
+            { NULL, NULL, NULL }
+        };
+
+        menu = buttons->priv->const_menu = gtk_menu_new();
+        gtk_menu_set_reserve_toggle_size(GTK_MENU(menu), FALSE);
+
+        for (i = 0; constants[i].name != NULL; i++) {
+            GtkWidget *item;
+
+            item = gtk_menu_item_new_with_label(_(constants[i].name));
+            gtk_widget_set_tooltip_text(item, _(constants[i].tooltip));
+            g_object_set_data(G_OBJECT(item), "const", g_strdup(constants[i].constant));
+            gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+            g_signal_connect(item, "activate", G_CALLBACK(insert_const_cb), buttons);
+            gtk_widget_show(item);
+        }
+    }
+
+    popup_button_menu(widget, GTK_MENU(buttons->priv->const_menu));
+}
 
 void factorize_cb(GtkWidget *widget, MathButtons *buttons);
 G_MODULE_EXPORT
@@ -1136,7 +1199,7 @@ character_code_dialog_response_cb(GtkWidget *dialog, gint response_id, MathButto
 
     text = gtk_entry_get_text(GTK_ENTRY(buttons->priv->character_code_entry));
 
-    if (response_id == GTK_RESPONSE_OK) {     
+    if (response_id == GTK_RESPONSE_OK) {
         MPNumber x;
         int i = 0;
 
@@ -1240,7 +1303,7 @@ number_mode_changed_cb(MathEquation *equation, GParamSpec *spec, MathButtons *bu
 {
     GList *i;
     NumberMode mode;
-  
+
     mode = math_equation_get_number_mode(equation);
 
     for (i = buttons->priv->superscript_toggles; i; i = i->next) {
