@@ -248,7 +248,7 @@ static void about_cb(GtkWidget* widget, MathWindow* window)
            "GNU General Public License for more details."),
         N_("You should have received a copy of the GNU General Public License "
            "along with MATE Calculator; if not, write to the Free Software Foundation, Inc., "
-           "151 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA")
+           "51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.")
     };
 
     char *license_trans = g_strjoin ("\n\n", _(license[0]), _(license[1]), _(license[2]), NULL);
@@ -262,7 +262,7 @@ static void about_cb(GtkWidget* widget, MathWindow* window)
         "version", VERSION,
         "title", _("About MATE Calculator"),
         "copyright", _("Copyright \xc2\xa9 1986–2010 The GCalctool authors\n"
-                       "Copyright \xc2\xa9 2011-2019 MATE developers"),
+                       "Copyright \xc2\xa9 2011-2020 MATE developers"),
         "license", license_trans,
         "comments", _("Calculator with financial and scientific modes."),
         "authors", authors,
@@ -466,7 +466,7 @@ create_gui(MathWindow *window)
 
     vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 6);
     gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);
-    gtk_box_pack_start(GTK_BOX(main_vbox), vbox, TRUE, TRUE, 0);  
+    gtk_box_pack_start(GTK_BOX(main_vbox), vbox, TRUE, TRUE, 0);
     gtk_widget_show(vbox);
 
     scrolled_window = gtk_scrolled_window_new(NULL, NULL);
