@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2008-2011 Robert Ancell.
- * 
+ *
  * This program is free software: you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
  * Foundation, either version 2 of the License, or (at your option) any later
@@ -28,6 +28,8 @@ static int passes = 0;
 #  define  __attribute__(x)  /*NOTHING*/
 #endif
 
+#define test(X, Y, Z) Test(X, Y, Z, 9)
+
 static void pass(const char *format, ...) __attribute__((format(printf, 1, 2)));
 static void fail(const char *format, ...) __attribute__((format(printf, 1, 2)));
 
@@ -72,7 +74,7 @@ error_code_to_string(MPErrorCode error)
 
 
 static void
-test(char *expression, char *expected, int expected_error)
+Test(char *expression, char *expected, int expected_error, int trailing_digits)
 {
     MPErrorCode error;
     MPNumber result;
@@ -83,7 +85,7 @@ test(char *expression, char *expected, int expected_error)
         char *result_str;
         MpSerializer *serializer;
 
-        serializer = mp_serializer_new(MP_DISPLAY_FORMAT_FIXED, options.base, 9);
+        serializer = mp_serializer_new(MP_DISPLAY_FORMAT_FIXED, options.base, trailing_digits);
         result_str = mp_serializer_to_string(serializer, &result);
         g_object_unref(serializer);
 
@@ -123,25 +125,25 @@ test_conversions()
     options.wordlen = 32;
     options.angle_units = MP_DEGREES;
     options.convert = do_convert;
-  
+
     /* Angle units */
     //test("π radians in degrees", "180", 0);
     test("100 gradians in degrees", "90", 0);
 
     /* Length */
-    test("1 meter in mm", "1000", 0);  
+    test("1 meter in mm", "1000", 0);
     test("1m in mm", "1000", 0);
     test("1 inch in cm", "2.54", 0);
 
     /* Area */
     test("1m² in mm²", "1000000", 0);
-  
+
     /* Volume */
-    test("1m³ in mm³", "1000000000", 0);  
+    test("1m³ in mm³", "1000000000", 0);
 
     /* Weight */
     test("1 kg in pounds", "2.204622622", 0);
-  
+
     /* Duration */
     test("1 minute in seconds", "60", 0);
     test("1s in ms", "1000", 0);
@@ -174,7 +176,7 @@ get_variable(const char *name, MPNumber *z, void *data)
         mp_set_from_integer (3, z);
         return 1;
     }
-    return 0;  
+    return 0;
 }
 
 
@@ -189,7 +191,7 @@ test_equations()
     memset(&options, 0, sizeof(options));
     options.wordlen = 32;
     options.angle_units = MP_DEGREES;
-    options.variable_is_defined = variable_is_defined;  
+    options.variable_is_defined = variable_is_defined;
     options.get_variable = get_variable;
     options.set_variable = set_variable;
 
@@ -249,7 +251,7 @@ test_equations()
     test("١٢٣٤٥٦٧٨٩٠", "1234567890", 0);
     test("۱۲۳۴۵۶۷۸۹۰", "1234567890", 0);
 
-/*    
+/*
     //test("2A", "2000000000000000", 0);
     test("2T", "2000000000000", 0);
     test("2G", "2000000000", 0);
@@ -289,7 +291,7 @@ test_equations()
     test("2y", "6", 0);
     test("y2", "", PARSER_ERR_INVALID);
     test("y 2", "", PARSER_ERR_INVALID);
-    test("2z", "", PARSER_ERR_UNKNOWN_VARIABLE);  
+    test("2z", "", PARSER_ERR_UNKNOWN_VARIABLE);
     test("z2", "", PARSER_ERR_UNKNOWN_VARIABLE);
     test("z 2", "", PARSER_ERR_UNKNOWN_VARIABLE);
     test("z(2)", "", PARSER_ERR_UNKNOWN_VARIABLE);
@@ -313,8 +315,21 @@ test_equations()
     test("2x²+2xy−12y²", "−88", 0);
 
     test("π", "3.141592654", 0);
+    test("pi", "3.141592654", 0);
     test("e", "2.718281828", 0);
 
+    /* Physical constants */
+
+    test("c₀", "299792458", 0);
+    Test("μ₀", "0.0000012566370614", 0, 16);
+    Test("ε₀", "0.00000000000885418782", 0, 20);
+    Test("G",  "0.0000000000667408", 0, 16);
+    Test("h",  "0.000000000000000000000000000000000662607004", 0, 42);
+    Test("ｅ", "0.00000000000000000016021766208", 0, 29);
+    Test("mₑ", "0.000000000000000000000000000000910938356", 0, 39);
+    Test("mₚ", "0.000000000000000000000000001672621898", 0, 36);
+    test("Nₐ", "602214086000000000000000", 0);
+
     test("z=99", "99", 0);
     test("longname=99", "99", 0);
     //test("e=99", "", PARSER_ERR_BUILTIN_VARIABLE);
@@ -364,6 +379,38 @@ test_equations()
     test("(1+2×3)", "7", 0);
     test("2(1+1)", "4", 0);
     test("4÷2(1+1)", "4", 0);
+    test ("1 + 2 - 3 * 4 / 5", "0.6", 0);
+    test ("20 / 10 mod 3", "2", 0);
+    test ("12 / 3 √4", "8", 0);
+    test ("√5!", "10.95445115", 0);
+    test ("4 ^ sin 30", "2", 0);
+    test ("4 ^ (sin 30)", "2", 0);
+    test ("4 ^ sin (30)", "2", 0);
+    //test ("sin (30) ^ 4", "0.0625", 0);
+    //test ("sin 30 ^ 4", "0.0625", 0);
+    test ("sin (30 ^ 4)", "0", 0);
+    test ("-ln(1)", "0", 0);
+    test ("ln(-1)", "3.141592654i", 0);
+
+    test ("10 / - 2", "−5", 0);
+    test ("10 * - 2", "−20", 0);
+    test ("10 ^ -2", "0.01", 0);
+    test ("-10 ^ 2", "−100", 0);
+    test ("sin (-30)", "−0.5", 0);
+    test ("sin - 30", "−0.5", 0);
+
+    test ("6 + 3!", "12", 0);
+    test ("4 * 3!", "24", 0);
+    test ("100 mod 3!", "4", 0);
+    test ("5! mod 7", "1", 0);
+    test ("24 / 3!", "4", 0);
+    test ("4! / 6", "4", 0);
+    test ("cos 5!", "−0.5", 0);
+    test ("sin 6!", "0", 0);
+    test ("- 4!", "−24", 0);
+    test ("3! ^ 3", "216", 0);
+    test ("3 ^ 3!", "729", 0);
+    //test ("(−√3)^2", "3", 0);
 
     /* Percentage */
     test("100%", "1", 0);
@@ -430,7 +477,7 @@ test_equations()
     test("8 mod 7", "1", 0);
     test("−1 mod 7", "6", 0);
 
-    test("sgn 0", "0", 0);  
+    test("sgn 0", "0", 0);
     test("sgn 3", "1", 0);
     test("sgn −3", "−1", 0);
     test("⌊3⌋", "3", 0);
@@ -546,16 +593,16 @@ test_equations()
     test("i", "i", 0);
     test("−i", "−i", 0);
     test("2i", "2i", 0);
-    test("1+i", "1+i", 0);  
+    test("1+i", "1+i", 0);
     test("i+1", "1+i", 0);
-    test("1−i", "1−i", 0);  
+    test("1−i", "1−i", 0);
     test("i−1", "−1+i", 0);
     test("i×i", "−1", 0);
     test("i÷i", "1", 0);
     test("1÷i", "−i", 0);
     test("|i|", "1", 0);
     test("|3+4i|", "5", 0);
-    test("arg 0", "", PARSER_ERR_MP);  
+    test("arg 0", "", PARSER_ERR_MP);
     test("arg 1", "0", 0);
     test("arg (1+i)", "45", 0);
     test("arg i", "90", 0);
@@ -569,6 +616,7 @@ test_equations()
     test("(−1)^0.5", "i", 0);
     test("√−4", "2i", 0);
     test("e^iπ", "−1", 0);
+    test("ln (e^iπ)", "3.141592654i", 0);
     test("log (−10) − (1 + πi÷ln(10))", "0", 0);
     test("ln (−e) − (1 + πi)", "0", 0);
     test("sin(iπ÷4) − i×sinh(π÷4)", "0", 0);
@@ -581,9 +629,9 @@ test_equations()
     test("1 and 1", "1", 0);
     test("3 and 5", "1", 0);
 
-    test("0 or 0", "0", 0);  
+    test("0 or 0", "0", 0);
     test("1 or 0", "1", 0);
-    test("0 or 1", "1", 0);  
+    test("0 or 1", "1", 0);
     test("1 or 1", "1", 0);
     test("3 or 5", "7", 0);
 
